﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Forms;
using System.Windows.Media.Imaging;

using Autodesk.Revit.DB;
using Autodesk.Revit.UI;
using RApplication = Autodesk.Revit.ApplicationServices.Application;

namespace AddInIntegrationSample
{
    public class EApplication : IExternalApplication
    {

        static String addinAssmeblyPath = typeof(EApplication).Assembly.Location;

        void createCommandBinding(UIControlledApplication application)
        {
           RevitCommandId wallCreate = RevitCommandId.LookupCommandId("ID_OBJECTS_WALL");
           AddInCommandBinding binding = application.CreateAddInCommandBinding(wallCreate);
           binding.Executed += new EventHandler<Autodesk.Revit.UI.Events.ExecutedEventArgs>(binding_Executed);
           binding.CanExecute += new EventHandler<Autodesk.Revit.UI.Events.CanExecuteEventArgs>(binding_CanExecute);
        }


        BitmapSource convertFromBitmap(System.Drawing.Bitmap bitmap)
        {
            return System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                bitmap.GetHbitmap(),
                IntPtr.Zero,
                Int32Rect.Empty,
                BitmapSizeOptions.FromEmptyOptions());
        }

        void createRibbonButton(UIControlledApplication application)
        {
            application.CreateRibbonTab("AddIn Integration");
            RibbonPanel rp = application.CreateRibbonPanel("AddIn Integration", "Testing");

            PushButtonData pbd = new PushButtonData("Wall", "Goto WikiHelp for wall creation",
                    addinAssmeblyPath,
                    "AddInIntegrationSample.CalcCommand");
            ContextualHelp ch = new ContextualHelp(ContextualHelpType.ContextId, "ID_OBJECTS_WALL");
            pbd.SetContextualHelp(ch);
            pbd.LongDescription = "We redirect the wiki help for this button to Wall creation.";
            pbd.LargeImage = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall);
            pbd.Image = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall_S);
            
            PushButton pb = rp.AddItem(pbd) as PushButton;
            pb.Enabled = true;
            pb.AvailabilityClassName = "AddInIntegrationSample.ApplicationAvailabilityClass";

            PushButtonData pbd1 = new PushButtonData("GotoGoogle", "Go to Google",
                    addinAssmeblyPath,
                    "AddInIntegrationSample.CalcCommand");
            ContextualHelp ch1 = new ContextualHelp(ContextualHelpType.Url, "http://www.google.com/");
            pbd1.SetContextualHelp(ch1);
            pbd1.LongDescription = "Go to google.";
            pbd1.LargeImage = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall);
            pbd1.Image = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall_S);
            PushButton pb1 = rp.AddItem(pbd1) as PushButton;
            pb1.AvailabilityClassName = "AddInIntegrationSample.ApplicationAvailabilityClass";

            PushButtonData pbd2 = new PushButtonData("GotoRevitAPIHelpFile", "Go to RevitAPI what's new",
                addinAssmeblyPath,
                "AddInIntegrationSample.CalcCommand");
            ContextualHelp ch2 = new ContextualHelp(ContextualHelpType.ChmFile, @"C:\Saikat\Revit SDKs\2013\RTM\Software Development Kit\RevitAPI.chm");
            ch2.HelpTopicUrl = @"What's New.html";
            pbd2.SetContextualHelp(ch2);
            pbd2.LongDescription = "Go to Revit API what's new.";
            pbd2.LargeImage = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall);
            pbd2.Image = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall_S);
            PushButton pb2 = rp.AddItem(pbd2) as PushButton;
            pb2.AvailabilityClassName = "AddInIntegrationSample.ApplicationAvailabilityClass";


            PushButtonData pbd3 = new PushButtonData("PreviewControl", "Preview all views",
                addinAssmeblyPath,
                "AddInIntegrationSample.PreviewCommand");
            pbd3.LargeImage = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall);
            pbd3.Image = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall_S);
            PushButton pb3 = rp.AddItem(pbd3) as PushButton;
            pb3.AvailabilityClassName = "AddInIntegrationSample.ApplicationAvailabilityClass";


            PushButtonData pbd5 = new PushButtonData("Drag_And_Drop", "Drag and Drop", addinAssmeblyPath,
                                                     "AddInIntegrationSample.DragAndDropCommand");
            pbd5.LargeImage = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall);
            pbd5.Image = convertFromBitmap(AddInIntegrationSample.Properties.Resources.StrcturalWall_S);
            PushButton pb5 = rp.AddItem(pbd5) as PushButton;
            pb5.AvailabilityClassName = "AddInIntegrationSample.ApplicationAvailabilityClass";
        }


        public Result OnShutdown(UIControlledApplication application)
        {
            return Result.Succeeded;
        }

        public Result OnStartup(UIControlledApplication application)
        {
            s_uiApplication = application;
            ApplicationOptions.Initialize(this);

            createCommandBinding(application);
            createRibbonButton(application);
            
            // add custom tabs to options dialog.
            AddTabCommand addTabCommand = new AddTabCommand(application);
            addTabCommand.AddTabToOptionsDialog();
            return Result.Succeeded;
        }

        public UIControlledApplication UIControlledApplication
        {
            get { return s_uiApplication; }
        }

        private UIControlledApplication s_uiApplication;

        void binding_CanExecute(object sender, Autodesk.Revit.UI.Events.CanExecuteEventArgs e)
        {
            e.CanExecute = true;
        }

        void binding_Executed(object sender, Autodesk.Revit.UI.Events.ExecutedEventArgs e)
        {
            UIDocument uiDocument = new UIDocument(e.ActiveDocument);
            Reference reference = uiDocument.Selection.PickObject(Autodesk.Revit.UI.Selection.ObjectType.Element);

            Transaction tran = new Transaction(e.ActiveDocument, "Del");
            tran.Start();

            e.ActiveDocument.Delete(reference.ElementId);

            tran.Commit();
        }
    }

    [Autodesk.Revit.Attributes.Transaction(Autodesk.Revit.Attributes.TransactionMode.Manual)]
    public class CalcCommand : IExternalCommand
    {

        public Result Execute(ExternalCommandData commandData, ref string message, ElementSet elements)
        {
            TaskDialog.Show("Dummy command", "This is a dummy command for buttons associated to contextual help.");
            return Result.Succeeded;
        }

    }

    [Autodesk.Revit.Attributes.Transaction(Autodesk.Revit.Attributes.TransactionMode.Manual)]
    public class DelCommand : IExternalCommand
    {

        public Result Execute(ExternalCommandData commandData, ref string message, ElementSet elements)
        {
            UIDocument uiDoc = commandData.Application.OpenAndActivateDocument(@"C:\Users\xiap\Documents\building.rvt");
            if (uiDoc == null)
                return Result.Failed;

            _document = uiDoc.Document;

            Reference reference = commandData.Application.ActiveUIDocument.Selection.PickObject(Autodesk.Revit.UI.Selection.ObjectType.Element);

            Transaction tran = new Transaction(_document, "Del");
            tran.Start();

            _document.Delete(reference.ElementId);

            tran.Commit();
            return Result.Succeeded;
        }


        private Document _document = null;
    }








}
